/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.explorer.view.impl;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import javax.annotation.Nonnull;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;

/***********************************************************************************************************************
 *
 * When nb.useSwingHtmlRendering=true, that is HtmlLabelUI is not used, the selection colors are not ok. This class
 * fixes them. This class provides a simple way to renderer a text property near the node.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Slf4j
public class PatchedListNodeRenderer extends JPanel implements ListCellRenderer
  {
    @Nonnull
    private final ListCellRenderer delegate;
    
    private Component component;
    
    @Getter @Setter
    private Color backgroundNonSelectionColor = Color.BLACK;
    
    @Getter @Setter
    private Color backgroundSelectionColor = Color.WHITE;
    
    @Getter @Setter
    private Color textNonSelectionColor = Color.WHITE;
    
    @Getter @Setter
    private Color textSelectionColor = Color.BLACK;

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    public PatchedListNodeRenderer (final @Nonnull ListCellRenderer delegate)
      {
        this.delegate = delegate;
        setLayout(new BorderLayout());
        setOpaque(false);
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Component getListCellRendererComponent (final @Nonnull JList list,
                                                   final @Nonnull Object object,
                                                   final int index,
                                                   final boolean selected,
                                                   final boolean hasFocus)
      {
        final Component newComponent = delegate.getListCellRendererComponent(list, object, index, selected, hasFocus);

        if (newComponent != component)
          {
            if (component != null)
              {
                remove(component);
              }

            add(component = newComponent, BorderLayout.CENTER);
//            final Font font = component.getFont();
//            smallFont = font.deriveFont(0.8f * font.getSize2D()); // FIXME: cache it?
          }

        component.setBackground(selected ? backgroundSelectionColor : backgroundNonSelectionColor);
        component.setForeground(selected ? textSelectionColor : textNonSelectionColor);

        return this;
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Dimension getSize()
      {
        if (component == null)
          {
            return super.getSize();
          }

        return component.getSize();
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Dimension getPreferredSize()
      {
        if (component == null)
          {
            return super.getPreferredSize();
          }

        return component.getPreferredSize();
      }
  }

